﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("IntegerParameterConfiguration", "")]
  [StorableClass]
  public class IntegerParameterConfiguration : ParameterConfiguration {
    [Storable]
    private IntValue length;
    public IntValue Length {
      get { return length; }
      set {
        if (length == value) return;
        length = value;
        OnParameterConfigurationChanged();
      }
    }

    [Storable]
    private IntMatrix bounds;
    public IntMatrix Bounds {
      get { return bounds; }
      set {
        if (bounds == value) return;
        bounds = value;
        OnParameterConfigurationChanged();
      }
    }

    [StorableConstructor]
    protected IntegerParameterConfiguration(bool deserializing) : base(deserializing) { }
    protected IntegerParameterConfiguration(IntegerParameterConfiguration original, Cloner cloner)
      : base(original, cloner) {
      length = cloner.Clone(original.length);
      bounds = cloner.Clone(original.bounds);
    }
    public IntegerParameterConfiguration(int length, int min, int max, int? step = null) {
      if (min >= max) throw new ArgumentException("min must be less than max", "min");
      if (step.HasValue && step.Value <= 0) throw new ArgumentException("step must be greater than zero or null", "step");
      this.length = new IntValue(length);
      bounds = new IntMatrix(1, step.HasValue ? 3 : 2);
      bounds[0, 0] = min;
      bounds[0, 1] = max;
      if (step.HasValue) bounds[0, 2] = step.Value;
    }
    public IntegerParameterConfiguration(int length, IList<int> min, IList<int> max, IList<int> step = null) {
      if (min.Count == 0) throw new ArgumentException("Bounds must be given for the integer parameters.");
      if (min.Count != max.Count) throw new ArgumentException("min must be of the same length as max", "min");
      if (step != null && min.Count != step.Count) throw new ArgumentException("step must be of the same length as min or null", "step");
      if (min.Zip(max, (mi, ma) => mi >= ma).Any()) throw new ArgumentException("min must be less than max in each dimension", "min");
      this.length = new IntValue(length);
      bounds = new IntMatrix(min.Count, step != null ? 3 : 2);
      for (int i = 0; i < min.Count; i++) {
        bounds[i, 0] = min[i];
        bounds[i, 1] = max[i];
        if (step != null) bounds[i, 2] = step[i];
      }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IntegerParameterConfiguration(this, cloner);
    }

  }
}
